@@ -26,7 +26,7 @@ class Backend(object):
         pass
 
     @abstractmethod
-    def create_app(self, name, desc):
+    def create_app(self, name, tags, desc):
         pass
 
     @abstractmethod
@@ -34,7 +34,7 @@ class Backend(object):
         pass
 
     @abstractmethod
-    def update_app(self, app_id, name, desc):
+    def update_app(self, app_id, name, tags, desc):
         pass
 
     @abstractmethod
@@ -74,15 +74,7 @@ class Backend(object):
         pass
 
     @abstractmethod
-    def update_model(self, model_id):
-        pass
-
-    @abstractmethod
-    def delete_model(self, model_id):
-        pass
-
-    @abstractmethod
-    def update_model(self, model_id):
+    def update_model(self, model_id, app_id, name, desc):
         pass
 
     @abstractmethod
