@@ -18,12 +18,13 @@ from waveletai.model import Model
 from contextlib import closing
 import requests
 from waveletai.envs import CHUNK_SIZE, WARN_SIZE
-from multiprocessing.dummy import Pool as ThreadPool
+from multiprocessing.dummy import Pool
 from waveletai.backend import Backend
 from waveletai.utils import with_api_exceptions_handler
 from waveletai.oauth import WaveletAIAuthenticator
 from waveletai.dataset import Dataset, Asset
 from waveletai.model import ModelVersion
+from waveletai.app import App
 from waveletai.exceptions import LoginFailed, NotADirectory, MissingNameOrPwdException, FileUploadException
 from waveletai.constants import DataType, ModelRegisterMode
 from waveletai.utils.storage_utils import UploadEntry, scan_upload_entries, SilentProgressIndicator, \
@@ -32,62 +33,223 @@ from waveletai.utils.datastream import FileStream, FileChunkStream, FileChunk
 
 _logger = logging.getLogger(__name__)
 
-pool = ThreadPool(9)
 
 
 class HostedBackend(Backend):
 
     @with_api_exceptions_handler
-    def create_app(self, name, desc):
-        pass
+    def create_app(self, name, tags, desc):
+        """
+        : 添加新应用
+        :param name str 应用名称
+        :param tags list 应用标签
+        :param desc str 应用描述
+        :return: Class `app` Object
+        """
+        res = self._session.post(url=f'{self.api_address}/application/app/',
+                                 json={"name": name, "tags": tags, "desc": desc})
+        if res.json()["message"]:
+            raise Exception(res.json()["message"])
+        data = res.json()['data']
+        return App(self, data['id'], data['name'], data['desc'], data['create_time'], data['create_user_id'],
+                   data['update_time'], data['update_user_id'])
 
     @with_api_exceptions_handler
     def get_app(self, app_id):
-        pass
+        """
+        : 获取应用信息
+        :param app_id str 应用id
+        :return: Class `app` Object
+        """
+        res = self._session.get(url=f'{self.api_address}/application/app/{app_id}')
+        if res.json()['message']:
+            raise Exception(res.json()['message'])
+        data = res.json()['data']
+        return App(self, data['id'], data['name'], data['desc'], data['create_time'], data['create_user_id'],
+                   data['update_time'], data['update_user_id'])
 
     @with_api_exceptions_handler
-    def update_app(self, app_id, name, desc):
-        pass
+    def update_app(self, app_id, name, tags, desc):
+        """
+        更新应用信息
+        :param desc:
+        :param tags:
+        :param name:
+        :param app_id str 应用id
+        :return: Class `app` Object
+        """
+        res = self._session.put(url=f'{self.api_address}/application/app/{app_id}',
+                                json={"name": name, "tags": tags, "desc": desc})
+        if res.json()["message"]:
+            raise Exception(res.json()["message"])
+        data = res.json()['data']
+        return App(self, data['id'], data['name'], data['desc'], data['create_time'], data['create_user_id'],
+                   data['update_time'], data['update_user_id'])
 
     @with_api_exceptions_handler
     def delete_app(self, app_id):
-        pass
+        """
+        : 删除应用
+        :param app_id str 应用id
+        :return:
+        """
+        res = self._session.delete(url=f'{self.api_address}/application/app/{app_id}')
+        if res.json()['message']:
+            raise Exception(res.json()['message'])
+        data = res.json()['code']
+        return data
+
+    @with_api_exceptions_handler
+    def list_models(self, app_id):
+        """
+        : 获取应用下模型对象列表
+        :param app_id str 应用id
+        :return: List of Class `Model` Object
+        """
+        res = self._session.get(url=f'{self.api_address}/training/model/?query={{"app_id": "{app_id}"}}')
+        if res.json()['message']:
+            raise Exception(res.json()['message'])
+        data = res.json()['data']
+        return data
 
     @with_api_exceptions_handler
     def update_dataset(self, dataset_id, name, zone, desc):
-        pass
+        """
+        更新数据集信息
+        :param desc: 数据集描述
+        :param zone: 数据集组
+        :param name: 数据集名字
+        :param dataset_id: 数据集id
+        :return: Class `Dataset` Object
+        """
+        res = self._session.put(url=f'{self.api_address}/data/dataset/{dataset_id}',
+                                json={"name": name, "zone": zone, "desc": desc})
+        if res.json()['message']:
+            raise Exception(res.json()['message'])
+        data = res.json()['data']
+        return data
 
     @with_api_exceptions_handler
     def delete_dataset(self, dataset_id):
-        pass
+        """
+        删除数据集
+        :param dataset_id:
+        :return:
+        """
+        res = self._session.delete(url=f'{self.api_address}/data/dataset/{dataset_id}')
+        if res.json()['message']:
+            raise Exception(res.json()['message'])
+        data = res.json()['data']
+        return data
 
     @with_api_exceptions_handler
-    def update_model(self, model_id):
-        pass
+    def update_model(self, model_id, app_id, name, desc=''):
+        """
+        更新模型信息
+        :param app_id:
+        :param model_id:
+        :param name:
+        :param desc:
+        :return:
+        """
+        res = self._session.put(url=f'{self.api_address}/training/model/{model_id}',
+                                json={"app_id": app_id, "name": name, "desc": desc})
+        if res.json()['message']:
+            raise Exception(res.json()['message'])
+        data = res.json()['data']
+        return data
 
     @with_api_exceptions_handler
     def delete_model(self, model_id):
-        pass
+        """
+        删除模型
+        :param model_id:
+        :return:
+        """
+        res = self._session.delete(url=f'{self.api_address}/training/model/{model_id}')
+        if res.json()['message']:
+            raise Exception(res.json()['message'])
+        data = res.json()['data']
+        return data
 
     @with_api_exceptions_handler
     def get_model_version(self, model_id, version):
-        pass
+        """
+        根据模型id，模型版本查询
+        :param model_id:
+        :param version:
+        :return:
+        """
+        res = self._session.get(url=f'{self.api_address}/training/model-repo/modelrepo/{model_id}?version={version}')
+        if res.json()['message']:
+            raise Exception(res.json()['message'])
+        dict = res.json()['data'][0]
+        return ModelVersion(self, dict['id'], dict['version'], dict['desc'], dict['model_id'], dict['model_name'],
+                            dict['mode'], dict['create_user_name'], dict['create_time'], dict['create_user_id'],
+                            dict['json_data'])
 
     @with_api_exceptions_handler
     def list_model_versions(self, model_id):
-        pass
+        """
+        根据模型id，模型版本查询
+        :param model_id:
+        :return:
+        """
+        res = self._session.get(url=f'{self.api_address}/training/model-repo/modelrepo/{model_id}')
+        if res.json()['message']:
+            raise Exception(res.json()['message'])
+        dict = res.json()['data']
+        datas = []
+        for d in dict:
+            datas.append(ModelVersion(self, d['id'], d['version'], d['desc'], d['model_id'], d['model_name'],
+                                      d['mode'], d['create_user_name'], d['create_time'], d['create_user_id'],
+                                      d['json_data']))
+        return datas
 
     @with_api_exceptions_handler
     def abandon_model_version(self, model_id, version):
-        pass
+        """
+        根据模型id，删除模型版本
+        :param model_id:
+        :param version:
+        :return:
+        """
+        res = self._session.delete(url=f'{self.api_address}/training/model-repo/modelrepo/{model_id}?version={version}')
+        if res.json()['message']:
+            raise Exception(res.json()['message'])
+        data = res.json()['data']
+        return data
 
     @with_api_exceptions_handler
     def update_model_version(self, model_id, version, desc):
-        pass
+        """
+        根据模型id，修改模型版本
+        :param desc:
+        :param model_id:
+        :param version:
+        :return:
+        """
+        res = self._session.put(url=f'{self.api_address}/training/model-repo/modelrepo/{model_id}?version={version}',
+                                json={"desc": desc})
+        if res.json()['message']:
+            raise Exception(res.json()['message'])
+        dict = res.json()['data']
+        return ModelVersion(self, dict['id'], dict['version'], dict['desc'], dict['model_id'], dict['model_name'],
+                            dict['mode'], dict['create_user_name'], dict['create_time'], dict['create_user_id'],
+                            dict['json_data'])
 
     @with_api_exceptions_handler
     def list_experiments(self, model_id):
-        pass
+        """
+        根据模型id，获取实验数据
+        :param model_id:
+        :return:
+        """
+        res = self._session.get(url=f'{self.api_address}/training/experiment/getexperiment/{model_id}')
+        if res.json()['message']:
+            raise Exception(res.json()['message'])
+        data = res.json()['data']
+        return data
 
     @with_api_exceptions_handler
     def download_model_artifact(self, model_id, path, destination):
@@ -98,8 +260,35 @@ class HostedBackend(Backend):
         pass
 
     @with_api_exceptions_handler
-    def list_model_releases(self, model_id, version):
-        pass
+    def list_model_releases(self, model_id, version=None):
+        """
+        根据模型id，版本，获取发布数据
+        :param version:
+        :param model_id:
+        :return:
+        """
+        if version is None:
+            res = self._session.get(url=f'{self.api_address}/training/model-release/getrelease/{model_id}')
+        else:
+            res = self._session.get(
+                url=f'{self.api_address}/training/model-release/getrelease/{model_id}?version={version}')
+        if res.json()['message']:
+            raise Exception(res.json()['message'])
+        data = res.json()['data']
+        return data
+
+    @with_api_exceptions_handler
+    def list_model_releases_by_repo(self, repo_id):
+        """
+        查询当前模型版本的发布服务列表
+        :param repo_id:
+        :return:
+        """
+        res = self._session.get(url=f'{self.api_address}/training/model-release/getrelease_byrepo/{repo_id}')
+        if res.json()['message']:
+            raise Exception(res.json()['message'])
+        data = res.json()['data']
+        return data
 
     @with_api_exceptions_handler
     def __init__(self, name=None, pwd=None):
@@ -358,27 +547,28 @@ class HostedBackend(Backend):
             if type == 'timeseries':
                 self._timeseries_download(dataset_id, name, destination)
             else:
-                urls = []
-                artifacts = self._list_dataset_artifacts(dataset_id)
-                if '.' not in path:
-                    resultinfo, dir = self._list_dir_artifacts(dataset_id, path, [], [])
-                    for i in dir:
-                        if not os.path.exists(destination + i):
-                            os.makedirs(destination + i)
-                    for asset in artifacts:
-                        for res in resultinfo:
-                            if asset.name in res:
-                                if '/' in res:
-                                    res_des = res.rsplit('/', 1)
-                                    res_des = res_des[0]
-                                urls.append((asset.path, destination + res_des, asset.name))
-                else:
-                    for asset in artifacts:
-                        if path in asset.path:
-                            urls.append((asset.path, destination, asset.name))
-                pool.map(self._url_download, urls)
-                pool.close()
-                pool.join()
+                with Pool(5) as pool:
+                    urls = []
+                    artifacts = self._list_dataset_artifacts(dataset_id)
+                    if '.' not in path:
+                        resultinfo, dir = self._list_dir_artifacts(dataset_id, path, [], [])
+                        for i in dir:
+                            if not os.path.exists(destination + i):
+                                os.makedirs(destination + i)
+                        for asset in artifacts:
+                            for res in resultinfo:
+                                if asset.name in res:
+                                    if '/' in res:
+                                        res_des = res.rsplit('/', 1)
+                                        res_des = res_des[0]
+                                    urls.append((asset.path, destination + res_des, asset.name))
+                    else:
+                        for asset in artifacts:
+                            if path in asset.path:
+                                urls.append((asset.path, destination, asset.name))
+                    pool.map(self._url_download, urls)
+                    pool.close()
+                    pool.join()
         except Exception as e:
             raise e
 
@@ -414,21 +604,22 @@ class HostedBackend(Backend):
             if type == 'timeseries':
                 self._timeseries_download(dataset_id, name, destination)
             else:
-                artifacts = self._list_dataset_artifacts(dataset_id)
-                resultinfo, dir = self._list_dir_artifacts(dataset_id=dataset_id, path=None, result=[], dir=[])
-                urls = []
-                for d in dir:
-                    if not os.path.exists(destination + d):
-                        os.makedirs(destination + d)
-                for asset in artifacts:
-                    for res in resultinfo:
-                        if res in asset.path:
-                            res_des = res.rsplit('/', 1)
-                            res_des = res_des[0]
-                            urls.append((asset.path, destination + res_des, asset.name))
-                pool.map(self._url_download, list(set(urls)))
-                pool.close()
-                pool.join()
+                with Pool(5) as pool:
+                    artifacts = self._list_dataset_artifacts(dataset_id)
+                    resultinfo, dir = self._list_dir_artifacts(dataset_id=dataset_id, path=None, result=[], dir=[])
+                    urls = []
+                    for d in dir:
+                        if not os.path.exists(destination + d):
+                            os.makedirs(destination + d)
+                    for asset in artifacts:
+                        for res in resultinfo:
+                            if res in asset.path:
+                                res_des = res.rsplit('/', 1)
+                                res_des = res_des[0]
+                                urls.append((asset.path, destination + res_des, asset.name))
+                    pool.map(self._url_download, list(set(urls)))
+                    pool.close()
+                    pool.join()
         except Exception as e:
             raise e
 
@@ -499,8 +690,27 @@ class HostedBackend(Backend):
             return e
 
     @with_api_exceptions_handler
-    def create_model(self, app_id, name, desc):
-        pass
+    def create_model(self, app_id, name, desc='', auth_id='', git_url='', http_url=''):
+        """
+        当前应用下创建模型
+        :param app_id: 应用ip
+        :param http_url: http地址 默认空
+        :param auth_id: auth_id 默认空
+        :param git_url: git地址 默认空
+        :param name:模型名称
+        :param desc:模型备注
+        :return: List of Class `Model` Object
+        """
+        res = self._session.post(url=f'{self.api_address}/training/model/', json={"app_id": app_id,
+                                                                                  "name": name,
+                                                                                  "desc": desc,
+                                                                                  "auth_id": auth_id,
+                                                                                  "git_url": git_url,
+                                                                                  "http_url": http_url})
+        if res.json()['message']:
+            raise Exception(res.json()['message'])
+        data = res.json()['data']
+        return data
 
     @with_api_exceptions_handler
     def get_model(self, model_id):
@@ -547,7 +757,8 @@ class HostedBackend(Backend):
         )
         dict = res.json()['data']
         return ModelVersion(self, dict['id'], dict['version'], dict['desc'], dict['model_id'], dict['model_name'],
-                            dict['mode'], dict['create_user_name'], dict['create_time'], dict['create_user_id'])
+                            dict['mode'], dict['create_user_name'], dict['create_time'], dict['create_user_id'],
+                            dict['json_data'])
 
     @with_api_exceptions_handler
     def download_model(self, model_id, version, destination):
@@ -650,3 +861,134 @@ class HostedBackend(Backend):
         response = session.send(session.prepare_request(request))
         response.raise_for_status()
         return response
+
+    @with_api_exceptions_handler
+    def download_model_version_asset(self, repo_id, path, destination=''):
+        if not destination:
+            destination = os.getcwd()
+
+        if not os.path.exists(destination):
+            os.makedirs(destination)
+        elif not os.path.isdir(destination):
+            raise NotADirectory(destination)
+
+        try:
+            urls = []
+            minio_path = self.api_address.rsplit('/', 1)
+            minio_path = minio_path[0] + '/resource'
+            s3path = self._get_repo_s3path(repo_id)
+            if s3path is None:
+                return 's3path不存在'
+            with Pool(5) as pool:
+                if '.' not in path:
+                    resultinfo, dir = self.list_dir_model_artifacts(repo_id, path, [], [])
+                    for i in dir:
+                        if i[0] != '/':
+                            continue
+                        if not os.path.exists(destination + i):
+                            os.makedirs(destination + i)
+                    for result in resultinfo:
+                        result_path = result.rsplit('/', 1)
+                        des_path = result_path[0]
+                        result_path = result_path[-1]
+                        file_path = minio_path + s3path + '/' + result_path
+                        urls.append((file_path, destination + '/' + des_path, result_path))
+                else:
+                    if '/' in path:
+                        path = path.rsplit('/', 1)
+                        des_path = path[0]
+                        path = path[-1]
+                        file_path = minio_path + s3path + path
+                        urls.append((file_path, destination + '/' + des_path, path))
+                    else:
+                        file_path = path + s3path + path
+                        urls.append((file_path, destination, path))
+                pool.map(self._url_download, urls)
+                pool.close()
+                pool.join()
+        except Exception as e:
+            raise e
+
+    @with_api_exceptions_handler
+    def download_model_version_artifacts(self, repo_id, destination=''):
+        if not destination:
+            destination = os.getcwd()
+
+        if not os.path.exists(destination):
+            os.makedirs(destination)
+        elif not os.path.isdir(destination):
+            raise NotADirectory(destination)
+
+        try:
+            urls = []
+            minio_path = self.api_address.rsplit('/', 1)
+            minio_path = minio_path[0] + '/resource'
+            s3path = self._get_repo_s3path(repo_id)
+            if s3path is None:
+                return 's3path不存在'
+            with Pool(5) as pool:
+                resultinfo, dir = self._list_dir_model_artifacts(repo_id, None, [], [])
+                for i in dir:
+                    if i[0] != '/':
+                        continue
+                    if not os.path.exists(destination + i):
+                        os.makedirs(destination + i)
+                for result in resultinfo:
+                    result_path = result.rsplit('/', 1)
+                    des_path = result_path[0]
+                    result_path = result_path[-1]
+                    file_path = minio_path + s3path + '/' + result_path
+                    urls.append((file_path, destination + '/' + des_path, result_path))
+                pool.map(self._url_download, urls)
+                pool.close()
+                pool.join()
+        except Exception as e:
+            raise e
+
+    @with_api_exceptions_handler
+    def _list_dir_model_artifacts(self, repo_id, path, result, dir):
+        """
+        递归获取所有的文件，文件夹
+        :param repo_id:
+        :path 文件路径
+        :return: 返回文件，图片数据集文件，文件夹列表
+        """
+        res = self._session.get(url=f'{self.api_address}/training/run/{repo_id}/artifacts/list?path={path}')
+        if not path:
+            res = self._session.get(url=f'{self.api_address}/training/run/{repo_id}/artifacts/list')
+        if res.json()["message"]:
+            raise Exception(res.json()["message"])
+        for d in res.json()['data']:
+            if d['isDir'] is True:
+                dir.append(d['path'])
+                self._list_dir_model_artifacts(repo_id, d['path'], result, dir)
+            else:
+                if '/' in d['path']:
+                    dir_path = d['path'].split('/')
+                    dir_path.remove(dir_path[-1])
+                    dp = ''
+                    for p in dir_path:
+                        dp = dp + '/' + p
+                    dir.append(dp)
+                result.append(d['path'])
+        return list(set(result)), list(set(dir))
+
+    @with_api_exceptions_handler
+    def _get_repo_s3path(self, repo_id):
+        res = self._session.get(url=f'{self.api_address}/training/model-repo/{repo_id}/')
+        if res.json()["message"]:
+            raise Exception(res.json()["message"])
+        data = res.json()['data']
+        s3path = ''
+        if data['json_data'] is not None:
+            s3path = data['json_data']['model_uri']
+        return s3path
+
+    @with_api_exceptions_handler
+    def get_repo_by_repo_id(self, repo_id):
+        res = self._session.get(url=f'{self.api_address}/training/model-repo/{repo_id}/')
+        if res.json()["message"]:
+            raise Exception(res.json()["message"])
+        data = res.json()['data']
+        return ModelVersion(self, data['id'], data['version'], data['desc'], data['model_id'], data['model_name'],
+                            data['create_user_name'], data['create_time'], data['create_user_id'], data['mode'])
