@@ -12,13 +12,20 @@
 """{"api_url":"http://ai.xiaobodata.com","api_token":"","api_key":"1e1cad40-7605-4a64-bd74-151226ddc53a"}"""
 from enum import Enum, unique
 
-
 ANONYMOUS = 'ANONYMOUS'
 
 ANONYMOUS_API_TOKEN = 'eyJhcGlfYWRkcmVzcyI6Imh0dHBzOi8vdWkubmVwdHVuZS5haSIsImFwaV91cmwiOiJodHRwczovL3VpLm5lcHR1bmUuYW' \
                       'kiLCJhcGlfa2V5IjoiYjcwNmJjOGYtNzZmOS00YzJlLTkzOWQtNGJhMDM2ZjkzMmU0In0='
 
 
+@unique
+class ModelRegisterMode(Enum):
+    # 注册模型类型
+    DOCKER = 'docker-container'  # 镜像方式注册
+    FRAMEWORK = 'framework'  # 框架无代码部署
+    PYFUNC = 'py-func'  # 自定义模型注册
+
+
 @unique
 class DataType(Enum):
     # 文件类型 当前指csv
@@ -32,4 +39,4 @@ class DataType(Enum):
     # 暂未用到
     TYPE_TF_RECORD = "tf_record"
     # 暂未用到
-    TYPE_AUDIO = "audio"
\ No newline at end of file
+    TYPE_AUDIO = "audio"
