@@ -32,32 +32,22 @@ class Dataset(object):
         self.update_time = update_time
         self.update_user_id = update_user_id
 
-    def get(self):
-        """
-        # TODO()
-        返回当前数据集信息
-        :return: Class of `Dataset` Object
-        """
-        pass
-
     def update(self, name, zone, desc):
         """
-        # TODO()
         更新当前数据集信息
         :param: name 数据集名称
         :param: zone 数据集区域
         :param: desc 数据集参数
         :return: Class of `Dataset` Object
         """
-        pass
+        return self._backend.update_dataset(self.id, name, zone, desc)
 
     def delete(self):
         """
-        # TODO()
         删除当前数据集
         :return:
         """
-        pass
+        return self._backend.delete_dataset(self.id)
 
     def download_artifact(self, path, destination_dir=None):
         """Download an artifact (file) from the dataset.
